/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.financial.erpcredit.service;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.sg.audit.information.entity.Information;
import com.jeeplus.modules.sg.audit.information.service.InformationService;
import com.jeeplus.modules.sg.financial.erpcredit.entity.*;
import com.jeeplus.modules.sg.financial.erpcredit.mapper.ErpCreditEquipmentMapper;
import com.jeeplus.modules.sg.financial.erpcredit.mapper.ErpCreditMapper;
import com.jeeplus.modules.sg.financial.erpcredit.mapper.ErpCreditMaterialMapper;
import com.jeeplus.modules.sg.financial.erpcredit.util.ErpInfo;
import com.jeeplus.modules.sg.financial.erpcredit.util.ExportTemplate;
import com.jeeplus.modules.sg.financial.erpcredit.util.ExportUtil2;
import com.jeeplus.modules.sg.financial.settlement.entity.MaintainData;
import com.jeeplus.modules.sg.financial.settlement.mapper.DataMaintenanceMapper;
import com.jeeplus.modules.sg.financial.settlement.service.DataMaintenanceService;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Service
@Transactional(readOnly = true)
public class ErpCreditService extends CrudService<ErpCreditMapper,ErpCredit> {
    @Autowired
    private ErpCreditEquipmentMapper erpCreditEquipmentMapper;
    @Autowired
    private ErpCreditMaterialMapper erpCreditMaterialMapper;
    @Autowired
    private ErpCreditMapper erpCreditMapper;
    @Autowired
    private DataMaintenanceMapper dataMaintenanceMapper;
    @Autowired
    private InformationService informationService;
    @Autowired
    private DataMaintenanceService dataMaintenanceService;

    public List<ErpCredit> getListErpCredit(List<String> itemId,String status){
        List<ErpCredit> erpCreditList = new ArrayList<ErpCredit>();
        List<ErpCredit> erpCredits = erpCreditMapper.findErpCreditList(itemId);
        for (ErpCredit erpCredit:erpCredits){
            if (erpCredit.getStatus().equals(status)){
                erpCreditList.add(erpCredit);
            }
        }
        List<String> strings = new ArrayList<String>();
        for (ErpCredit erpCredit:erpCreditList){
            strings.add(erpCredit.getId());
        }
        List<ErpCreditEquipment> erpCreditChild1List = erpCreditEquipmentMapper.findErpCreditChild1List(strings);
        List<ErpCreditMaterial> erpCreditChild2List = erpCreditMaterialMapper.findErpCreditChild2List(strings);
        if (null!=erpCreditList&&erpCreditList.size()>0){
            for (ErpCredit erpCredit:erpCreditList){
                if (null!=erpCreditChild1List&&erpCreditChild1List.size()>0){
                    for (ErpCreditEquipment erpCreditEquipment:erpCreditChild1List){
                        if (erpCredit.getId().equals(erpCreditEquipment.getErpCredit().getId())){
                            erpCredit.getErpCreditChild1s().add(erpCreditEquipment);
                        }
                    }
                }
                if (null!=erpCreditChild2List&&erpCreditChild2List.size()>0){
                    for (ErpCreditMaterial erpCreditChild2:erpCreditChild2List){
                        if (erpCredit.getId().equals(erpCreditChild2.getErpCredit().getId())){
                            erpCredit.getErpCreditChild2s().add(erpCreditChild2);
                        }
                    }
                }
            }
        }
        return erpCreditList;
    }
    public ErpCredit get(String id) {
        ErpCredit erpCredit = super.get(id);
        erpCredit.setErpCreditChild1s(erpCreditEquipmentMapper.findList(new ErpCreditEquipment(erpCredit)));
        erpCredit.setErpCreditChild2s(erpCreditMaterialMapper.findList(new ErpCreditMaterial(erpCredit)));
//        erpCredit.setErpCreditChild3s(erpCreditChild3Mapper.findList(new ErpCreditChild3(erpCredit)));
        return erpCredit;
    }

    public List<ErpCredit> findList(ErpCredit erpCredit) {
        return super.findList(erpCredit);
    }

    public Page<ErpCredit> findPage(Page<ErpCredit> page, ErpCredit erpCredit) {
        return super.findPage(page, erpCredit);
    }

    @Transactional(readOnly = false)
    public void save(ErpCredit erpCredit,String string) {
        erpCredit.setStatus(string);
        super.save(erpCredit);
        for (ErpCreditEquipment erpCreditChild1 : erpCredit.getErpCreditChild1s()){
            if (ErpCreditEquipment.DEL_FLAG_NORMAL.equals(erpCredit.getDelFlag())){
                if (StringUtils.isBlank(erpCreditChild1.getId())) {
                    erpCreditChild1.setErpCredit(erpCredit);
                    erpCreditChild1.preInsert();
                    erpCreditEquipmentMapper.insert(erpCreditChild1);
                }else {
                    erpCreditChild1.preUpdate();
                    erpCreditEquipmentMapper.update(erpCreditChild1);
                }
            }else {
                erpCreditEquipmentMapper.delete(erpCreditChild1);
            }
        }
        for (ErpCreditMaterial erpCreditChild2 : erpCredit.getErpCreditChild2s()){
            if (ErpCreditMaterial.DEL_FLAG_NORMAL.equals(erpCreditChild2.getDelFlag())){
                if (StringUtils.isBlank(erpCreditChild2.getId())){
                    erpCreditChild2.setErpCredit(erpCredit);
                    erpCreditChild2.preInsert();
                    erpCreditMaterialMapper.insert(erpCreditChild2);
                }else {
                    erpCreditChild2.preUpdate();
                    erpCreditMaterialMapper.update(erpCreditChild2);
                }
            }else {
                erpCreditMaterialMapper.delete(erpCreditChild2);
            }
        }
//        for (ErpCreditChild3 erpCreditChild3 : erpCredit.getErpCreditChild3s()){
//            if (ErpCreditChild3.DEL_FLAG_NORMAL.equals(erpCreditChild3.getDelFlag())){
//                if (StringUtils.isBlank(erpCreditChild3.getId())){
//                    erpCreditChild3.setErpCredit(erpCredit);
//                    erpCreditChild3.preInsert();
//                    erpCreditChild3Mapper.insert(erpCreditChild3);
//                }else {
//                    erpCreditChild3.preUpdate();
//                    erpCreditChild3Mapper.update(erpCreditChild3);
//                }
//            }else {
//                erpCreditChild3Mapper.delete(erpCreditChild3);
//            }
//        }
    }

    @Transactional(readOnly = false)
    public void delete(ErpCredit erpCredit) {
        super.delete(erpCredit);
        erpCreditEquipmentMapper.delete(new ErpCreditEquipment(erpCredit));
        erpCreditMaterialMapper.delete(new ErpCreditMaterial(erpCredit));
//        erpCreditChild3Mapper.delete(new ErpCreditChild3(erpCredit));
    }

    //维护表
    public List<MaintainData> getFindListMain(List<String> strings){
        return dataMaintenanceMapper.findListMain(strings);
    }

    //导出
    public void getExport(String ids, HttpServletResponse response,String status){
        String[] strings = ids.split(",");
        List<String> strs = new ArrayList<String>();
        for (String str:strings){
            strs.add(str);
        }
        List<ErpCredit> erpCreditList = getListErpCredit(strs,status);
        List<MaintainData> maintainDatas = getFindListMain(strs);
        ExportTemplate exportTemplate = new ExportTemplate();
        File newFile = exportTemplate.createNewFile();
        // 新文件写入数据，并下载
        InputStream inputStream = null;
        XSSFWorkbook workbook = null;
        XSSFFont font=null;
        XSSFSheet sheetOverView = null;
        XSSFSheet sheetView = null;
        XSSFSheet sheetOtherFee = null;//定义其他费用明细表费用
        XSSFSheet sheetAssetsTotal= null;//资产总表
        XSSFSheet sheetLine1 = null;//移交生产总值一览表
        XSSFSheet sheetEquip=null;//移交总值设备表
        XSSFSheet sheetEquipMaterial=null;//设备材料费用
        XSSFSheet sheetSettlement=null;//结算表
        XSSFSheet sheetLine2=null;//线路2
        XSSFSheet sheetEquip2=null;//设备2
        try {
            inputStream = new FileInputStream(newFile);// 将excel文件转为输入流
            workbook = new XSSFWorkbook(inputStream);// 创建个workbook，
            font = workbook.createFont();
            sheetOverView =workbook.getSheetAt(0);//概述表
            sheetView = workbook.getSheetAt(1);//定义一览表
            sheetOtherFee = workbook.getSheetAt(2);//其他费用表
            sheetAssetsTotal = workbook.getSheetAt(3);//移交生产总值
            sheetLine1=workbook.getSheetAt(4);//线路1
            sheetEquip=workbook.getSheetAt(5);//设备1
            sheetEquipMaterial=workbook.getSheetAt(6);//设备材料清单表
            sheetSettlement=workbook.getSheetAt(7);//结算表
            sheetLine2=workbook.getSheetAt(9);//线路2
            sheetEquip2=workbook.getSheetAt(10);//设备2
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        try {
            // 写数据
            FileOutputStream fos = new FileOutputStream(newFile);
            ExportUtil2.getSheetOverView(sheetOverView,erpCreditList,maintainDatas);
            ExportUtil2.getSheetView(sheetView,erpCreditList,maintainDatas,workbook);//调用一览表写入方法
            ExportUtil2.getSheetOtherFee(sheetOtherFee,erpCreditList,maintainDatas);
            ExportUtil2.getSheetAssetsTotal(sheetAssetsTotal,erpCreditList);
            ExportUtil2.getSheetLine1(sheetLine1,erpCreditList,workbook);
            ExportUtil2.getSheetEquip(sheetEquip,erpCreditList,workbook);
            ExportUtil2.getSheetEquipMaterial(sheetEquipMaterial,erpCreditList,workbook);
            ExportUtil2.getSheetSettlement(sheetSettlement,erpCreditList,maintainDatas);
            ExportUtil2.getSheetLine2(sheetLine2,erpCreditList,workbook);
            ExportUtil2.getSheetEquip2(sheetEquip2,erpCreditList,workbook);
            workbook.write(fos);
            fos.flush();
            fos.close();
            InputStream fis = new BufferedInputStream(new FileInputStream(
                    newFile));
            byte[] buffer = new byte[fis.available()];
            fis.read(buffer);
            fis.close();
            response.reset();
            response.setContentType("text/html;charset=UTF-8");
            OutputStream toClient = new BufferedOutputStream(response.getOutputStream());
            response.setContentType("application/x-msdownload");
            String newName = URLEncoder.encode("竣工结算表" + System.currentTimeMillis() + ".xlsx","UTF-8");
            response.addHeader("Content-Disposition","attachment;filename=\"" + newName + "\"");
            response.addHeader("Content-Length", "" + newFile.length());
            toClient.write(buffer);
            toClient.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (null != inputStream) {
                    inputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        // 删除创建的新文件
        // this.deleteFile(newFile);
    }
    //判断条件
    public List<ErpJudge> getCheck(List<ErpCredit> erpCredits){
        List<ErpJudge> erpJudges = new ArrayList<ErpJudge>();
        ErpJudge erpJudge = null;
        Map<String,List> map = new  HashMap<String,List>();
        List<String> strings = new ArrayList<String>();
        for (ErpCredit erpCredit:erpCredits){
            strings.add(erpCredit.getItemId());
        }
        //得到物料是为完成数据
        List<Information> informationList = informationService.findNoFinishList(strings);
        List<String> maintainDataList = dataMaintenanceService.getJudge(strings);
        if (null!=informationList&&informationList.size()>0){
            for (Information information:informationList){
                erpJudge=new ErpJudge();
                erpJudge.setItemId(information.getProjectId());
                erpJudge.setMainFlag("有");
                erpJudge.setInfoFlag("否");
                erpJudges.add(erpJudge);
            }
        }
        if (null!=maintainDataList&&maintainDataList.size()>0){
            Boolean flag = true;
            for (String maintain:maintainDataList){
                flag=true;
                if (null!=erpCredits&&erpCredits.size()>0){
                    for (ErpJudge erpJudge1:erpJudges){
                       if (erpJudge1.getItemId().equals(maintain)){
                           erpJudge1.setMainFlag("无");
                           flag = false;
                           break;
                       }
                    }
                }
                if (flag){
                    erpJudge=new ErpJudge();
                    erpJudge.setItemId(maintain);
                    erpJudge.setMainFlag("无");
                    erpJudge.setInfoFlag("是");
                    erpJudges.add(erpJudge);
                }
            }
        }
        return erpJudges;
    }
}