/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.managementcenter.activiti.web;

import com.google.common.collect.Maps;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.act.service.ActProcessService;
import com.jeeplus.modules.act.service.ActTaskService;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.Construction;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.Disclose;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.UploadImages;
import com.jeeplus.modules.sg.managementcenter.activiti.service.ConstructionService;
import com.jeeplus.modules.sg.managementcenter.activiti.service.DiscloseService;
import com.jeeplus.modules.sg.managementcenter.activiti.service.UploadImagesService;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.activiti.engine.repository.ProcessDefinition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 施工交底Controller
 * @author
 * @version 2019-11-08
 */
@Controller
@RequestMapping(value = "${adminPath}/managementcenter/upload")

public class UploadController extends BaseController {

	@Autowired
	private ConstructionService constructionService;
	@Autowired
	private ActProcessService actProcessService;
	@Autowired
	private ActTaskService actTaskService;
	@Autowired
	private DiscloseService discloseService;
	@Autowired
	private UploadImagesService uploadImagesService;

	@ModelAttribute
	public UploadImages get(@RequestParam(required=false) String id) {
		UploadImages entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = uploadImagesService.get(id);
		}
		if (entity == null){
			entity = new UploadImages();
		}
		return entity;
	}

	@RequestMapping(value = "list")
	public String formList(UploadImages uploadImages, Model model) {
		model.addAttribute("uploadImages", uploadImages);
		return "modules/sg/managementcenter/activiti/upload";
	}

	@RequestMapping(value = "form")
	public String formUp(UploadImages uploadImages, Model model) {
		model.addAttribute("uploadImages", uploadImages);
		return "modules/sg/managementcenter/activiti/uploadList";
	}

	/**
	 * 图片管理列表页面
	 */
	@RequestMapping(value = {"images"})
	public String list(UploadImages uploadImages, Model model) {
		model.addAttribute("uploadImages", uploadImages);
		return "modules/sg/managementcenter/activiti/uploadList";
	}

	/**
	 * 查看，增加，编辑图片管理表单页面
	 */
	@RequestMapping(value = "imagesForm")
	public String form(UploadImages uploadImages, Model model) {
		model.addAttribute("uploadImages", uploadImages);
		return "modules/sg/managementcenter/activiti/uploadListForm";
	}


	/**
	 * 文件管理列表数据
	 */
	@ResponseBody
	@RequestMapping(value = "data")
	public Map<String, Object> data(UploadImages uploadImages, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<UploadImages> page = uploadImagesService.findPage(new Page<UploadImages>(request, response), uploadImages);
		//当判定系统为linux的时候
		if (Global.SYS_TYPE.equals("2")) {
			List<UploadImages> list = page.getList();
			for (UploadImages upload:
				 list) {
				upload.setPath("http://" + request.getServerName()  +upload.getPath());
			}
		}
		return getBootstrapData(page);
	}


	




	@ResponseBody
	@RequestMapping(value = "uploadSave")
	public AjaxJson uploadSave(UploadImages uploadImages, Model model,String pic) throws Exception{
		AjaxJson j = new AjaxJson();
		/**
		 * 后台hibernate-validation插件校验
		 */
		String errMsg = beanValidator(uploadImages);
		if (StringUtils.isNotBlank(errMsg)){
			j.setSuccess(false);
			j.setMsg(errMsg);
			return j;
		}

		/**
		 * 流程审批
		 */
		if (StringUtils.isBlank(uploadImages.getId())){
			uploadImages.setPath(pic);
			uploadImages.setProcInsId(uploadImages.getAct().getTaskId());
			uploadImages.setuId(UserUtils.getUser().getLoginName());
			uploadImagesService.save(uploadImages);
			//新增或编辑表单保存
			// 启动流程
			ProcessDefinition p = actProcessService.getProcessDefinition(uploadImages.getAct().getProcDefId());
			String title = uploadImages.getCurrentUser().getName()+"在"+ DateUtils.getDateTime()+"发起"+p.getName();
			actTaskService.startProcess(p.getKey(),  "xm_upolad_images", uploadImages.getId(), title);
			j.setMsg("提交审批成功!");
			j.getBody().put("targetUrl",  "/act/task/process/");
		}

		return j;
	}
	


}